#include <iostream>
#include <vector>
#include <cstdlib>
#include <algorithm>
using namespace std;
//Gene


struct Chromosome{
	vector<bool> values;

	int fitness() const{
		int rv =0;
		for(bool b: values){
			if(!b){
				rv += 1;
			}
		}
		return rv;
	}

	void init(int genes){
		values.clear();
		for(int i =0; i < genes; ++i){
			values.push_back(rand()%2 == 0);
		}
	}

	void mutate(double p){
		int l = (int)(p*100);
		for(int i =0; i < values.size(); ++i){
			int r = rand()%100;
			if(r < l){
				values[i] = !values[i];
			}
		}
	}	


	Chromosome crossover(Chromosome b){
		Chromosome rv;
		for(int i =0; i < values.size()/2; ++i){
			rv.values.push_back(values[i]);
		}
		for(int i = b.values.size()/2; i < b.values.size(); ++i){
			rv.values.push_back(b.values[i]);
		}
		return rv;
	}

	string asString(){
		string rv = "[ ";
		for(bool b: values){
			if(b){
				rv += "t ";
			}else{
				rv += "f ";
			}
		}
		rv += "]" + to_string(fitness());
		return rv;
	}

};

bool operator<(const Chromosome& lhs, const Chromosome& rhs){
	//magic
	return lhs.fitness() < rhs.fitness();
}



int main(){
	vector<Chromosome> pool;
	srand(time(0));
	for(int i=0; i < 20; ++i){
		Chromosome a;
		a.init(20);
		pool.push_back(a);
	}

	Chromosome best = pool[0];
	for(int n =0; n < 50; n++){
		sort(pool.begin(),pool.end());

		vector<Chromosome> children;
		for(int i =0; i < pool.size(); ++i){
			children.push_back(pool[i]);
		}

		for(int i = 0; i < pool.size()/2; ++i){
			//or just next
			Chromosome child = pool[i].crossover(pool[rand()%pool.size()]);
			children.push_back(child);
		}

		for(Chromosome& c: children){
			c.mutate(0.05);
		}

		sort(children.begin(),children.end());
		
		for(int i =0; i < pool.size(); ++i){
			pool[i] = children[i];
		}

		cout << "current generation" << endl;

		for(Chromosome c: pool){
			cout << c.asString() << endl;
		}
		if( pool[0] < best)
			best = pool[0];
	}
	cout << "the best..." << endl;
	cout << best.asString() << endl;

	return 0;
}
